; *******************************************************
; *** Character Dump - 25-Aug-2007 by Kevin Krausnick ***
; ***     Ported to Acme Assembler on 14-Sep-2024     ***
; *******************************************************

CHROUT    = $FFD2
GETIN     = $FFE4
SCREENRAM = $0400
PLOT      = $FFF0
CIACRA    = $DC0E
BGCOLOR   = 53281
CURSCOL   = 646

* = $801

;CREDITS & RUN LINE
!BYTE 32,8,10,0,158,50,48,56,50,58,143,32
!TEXT "BY KEVIN KRAUSNICK"
!BYTE 0,0,0

JSR INIT ;INITIALIZE

; *** MAIN PROGRAM LOOP ***
A1 JSR GETIN ;GET INPUT
BEQ A1
CMP #13
BEQ A2
CMP #32
BEQ A2
JSR PROCESSKEY
JSR NORMCHAR
JSR LEFTCHAR
JSR RIGHTCHAR
JMP A1
A2 JSR CLEANUP
RTS

; *** INITIALIZE ***
INIT=*
LDA #0
STA BGCOLOR
LDA CURSCOL
STA CURSAVE
LDY #0
I1 LDA INFO,Y
BEQ I2
JSR CHROUT
INY
JMP I1
I2 RTS

; *** PROCESS KEYPRESS ***
PROCESSKEY=*
STA KEYBYTE ;STORE KEYPRESS
JSR TITLEROW
LDA KEYBYTE
CMP #64
BCC L2
CMP #96
BCS L2
SEC
SBC #64
STA KEYBYTE

L2 LDA #0
STA $BB
LDA #$D0
STA $BC ;$D000 = START OF CHARACTER ROM
LDA 53272
AND #2
BEQ LA
LDA #$D8
STA $BC ;$D800 = LOWERCASE CHARACTER SET
LA LDY KEYBYTE
BEQ L4
L3 LDA $BB
CLC
ADC #8
STA $BB
LDA $BC
ADC #0
STA $BC
DEY
BNE L3 ;ROM OFFSET OF SELECTED CHARACTER

L4 LDA CIACRA
AND #$FE
STA CIACRA
LDA 1
AND #$FB
STA 1 ;UNLOCK CHARACTER ROM

LDY #0
L5 LDA ($BB),Y
STA ORIGINAL,Y
INY
CPY #8
BNE L5

LDA 1
ORA #4
STA 1
LDA CIACRA
ORA #1
STA CIACRA ;LOCK CHARACTER ROM
RTS

; *** TITLE THAT APPEARS ABOVE EACH CHARACTER BIT MATRIX ***
TITLEROW=*
LDY #0
T1 LDA TITLE,Y
BEQ T2
JSR CHROUT
INY
JMP T1
T2 RTS

; *** COPY CHARACTER MATRIX TO OUTPUT BUFFER ***
NORMCHAR=*
LDY #0
N1 LDA ORIGINAL,Y
STA MODIFIED,Y
INY
CPY #8
BNE N1
LDA #0
STA TAB
JSR SHOWCHAR
RTS

; *** ROTATE CHARACTER LEFT 90 DEGREES ***
LEFTCHAR=*
JSR CLEARMOD

LDY #0
LDA #$80
STA $BC
R0 LDA #1
STA $BB
LDX #0
R1 LDA ORIGINAL,Y
AND $BB
BEQ R2
LDA MODIFIED,X
ORA $BC
STA MODIFIED,X
R2 CLC
ROL $BB
INX
CPX #8
BNE R1
CLC
ROR $BC
INY
CPY #8
BNE R0
JSR SHOWCHAR
RTS

; *** ROTATE CHARACTER RIGHT 90 DEGREES ***
RIGHTCHAR=*
JSR CLEARMOD

LDY #0
LDA #1
STA $BC
R5 LDA #$80
STA $BB
LDX #0
R6 LDA ORIGINAL,Y
AND $BB
BEQ R7
LDA MODIFIED,X
ORA $BC
STA MODIFIED,X
R7 CLC
ROR $BB
INX
CPX #8
BNE R6
CLC
ROL $BC
INY
CPY #8
BNE R5
JSR SHOWCHAR
RTS

; *** CLEAR MODIFIED CHARACTER MATRIX ***
CLEARMOD=*
LDA #0
TAY
CM1 STA MODIFIED,Y ;CLEAR DESTINATION
INY
CPY #8
BNE CM1
RTS

; *** OUTPUT 8X8 BIT CHARACTER MATRIX ***
SHOWCHAR=*
LDY #8
LDA #145 ;UP
U1 JSR CHROUT
DEY
BNE U1

LDY #0

O1 TYA
PHA
SEC
JSR PLOT
LDY TAB
CLC
JSR PLOT
PLA
TAY

O3 LDA MODIFIED,Y
JSR SHOWBITS
LDA #32
JSR CHROUT
LDA #30 ;GREEN
JSR CHROUT
LDA MODIFIED,Y
JSR SHOWDECVAL
LDA #13
JSR CHROUT
INY
CPY #8
BNE O1

LDA TAB
CLC
ADC #13
STA TAB
RTS

; *** DISPLAY BINARY BIT PATTERN OF A SINGLE BYTE ***
SHOWBITS=*
TAX
TYA
PHA

LDY #8
TXA
L7 AND #$FF
BMI L8
LDA #151
JSR CHROUT
LDA #$30
JMP L9
L8 LDA #5
JSR CHROUT
LDA #$31
L9 JSR CHROUT
TXA
ASL
TAX
DEY
BNE L7

PLA
TAY
RTS

; *** SHOW DECIMAL VALUE OF BYTE***
SHOWDECVAL=*
PHA
LDA #'0'
STA DECVAL
STA DECVAL+1
STA DECVAL+2
PLA

P1 CMP #100
BCC P2
SEC
SBC #100
INC DECVAL
JMP P1
P2 CMP #10
BCC P3
SEC
SBC #10
INC DECVAL+1
JMP P2
P3 CMP #1
BCC P4
SEC
SBC #1
INC DECVAL+2
JMP P3

P4 LDA DECVAL
CMP #'0'
BNE P5
LDA #32
STA DECVAL
P5 JSR CHROUT
LDA DECVAL+1
CMP #'0'
BNE P6
LDA #32
CMP DECVAL
BEQ P6
LDA DECVAL+1
P6 JSR CHROUT
LDA DECVAL+2
JSR CHROUT
RTS

; *** CLEANUP BEFORE EXIT ***
CLEANUP=*
LDA CURSAVE
STA CURSCOL
RTS

; *** PROGRAM VARIABLES ***
KEYBYTE *=*+1 ;CBM ASCII VALUE OF KEYPRESS
ORIGINAL *=*+8 ;ORIGINAL CHARACTER ARRAY
MODIFIED *=*+8 ;MODIFIED CHARACTER ARRAY
CURSAVE *=*+1 ;SAVED CURSOR COLOR
DECVAL *=*+3 ;DECIMAL VALUE OF BYTE
TAB *=*+1 ;HORIZONTAL OFFSET FOR CHARACTER MATRIX

; *** DISPLAY STRINGS ***
INFO=*
!BYTE 13,129
!TEXT "PRESS A KEY TO SEE ITS BINARY BIT"
!BYTE 13
!TEXT "PATTERN, OR RETURN TO QUIT..."
!BYTE 13,0
TITLE=*
!BYTE 13,159
!TEXT "NORMAL   VAL -90 DEG  VAL +90 DEG  VAL"
!BYTE 13,17,17,17,17,17,17,17,17,0
